import { act, render, screen } from '@testing-library/react'
import userEvent from '@testing-library/user-event'
import { ERROR_EVENT } from 'cozy-harvest-lib/dist/models/flowEvents'
import { KonnectorError } from 'enums'
import React from 'react'
import { Provider } from 'react-redux'
import { accountsData } from 'tests/__mocks__/accountsData.mock'
import { createMockEcolyoStore } from 'tests/__mocks__/store'
import KonnectorModalFooter from './KonnectorModalFooter'

const mockedNavigate = jest.fn()
jest.mock('react-router-dom', () => ({
  ...jest.requireActual('react-router-dom'),
  useNavigate: () => mockedNavigate,
}))
const mockClose = jest.fn()
const mockDelete = jest.fn()
describe('KonnectorModalFooter component', () => {
  const store = createMockEcolyoStore()
  beforeEach(() => {
    jest.clearAllMocks()
  })

  it('should be rendered correctly', () => {
    const { container } = render(
      <Provider store={store}>
        <KonnectorModalFooter
          state={null}
          error={null}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={null}
          isUpdating={false}
        />
      </Provider>
    )
    expect(container).toMatchSnapshot()
  })

  it('should render "understood" for enedis sge when postal code not supported (USER_ACTION_NEEDED)', () => {
    render(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.USER_ACTION_NEEDED}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={null}
          isUpdating={false}
        />
      </Provider>
    )
    expect(
      screen.getByText('konnector_modal.button_understood')
    ).toBeInTheDocument()
  })

  it('should render "try again" for enedis sge when address / name / pdl are not matching (LOGIN_FAILED)', () => {
    render(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.LOGIN_FAILED}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={null}
          isUpdating={false}
        />
      </Provider>
    )
    expect(
      screen.getByText('konnector_modal.button_try_again')
    ).toBeInTheDocument()
  })

  it('should render "later" and "Verify infos" for enedis sge encountering an error on contract (TERMS_VERSION_MISMATCH)', () => {
    render(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.TERMS_VERSION_MISMATCH}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={null}
          isUpdating={false}
        />
      </Provider>
    )
    expect(screen.getByText('konnector_modal.button_later')).toBeInTheDocument()
    expect(
      screen.getByText('konnector_modal.button_check_info')
    ).toBeInTheDocument()
  })

  it('should go back to login', async () => {
    render(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.TERMS_VERSION_MISMATCH}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={accountsData[0]}
          isUpdating={true}
        />
      </Provider>
    )
    await act(async () => {
      await userEvent.click(screen.getByText('konnector_modal.button_go'))
    })
    expect(mockDelete).toHaveBeenCalled()
    expect(mockedNavigate).toHaveBeenCalledTimes(1)
  })
  it('should reset sge account and go back to login', async () => {
    render(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.TERMS_VERSION_MISMATCH}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={accountsData[0]}
          isUpdating={true}
        />
      </Provider>
    )
    await act(async () => {
      await userEvent.click(screen.getByText('konnector_modal.button_go'))
    })
    expect(mockDelete).toHaveBeenCalled()
    expect(mockedNavigate).toHaveBeenCalledTimes(1)
  })
  it('should not reset account if no account', async () => {
    render(
      <Provider store={store}>
        <KonnectorModalFooter
          state={ERROR_EVENT}
          error={KonnectorError.TERMS_VERSION_MISMATCH}
          handleCloseClick={mockClose}
          handleAccountDeletion={mockDelete}
          account={null}
          isUpdating={false}
        />
      </Provider>
    )
    await act(async () => {
      await userEvent.click(
        screen.getByText('konnector_modal.button_check_info')
      )
    })
    expect(mockedNavigate).toHaveBeenCalledTimes(1)
    expect(mockDelete).toHaveBeenCalledTimes(0)
  })
})
