import { QueryResult } from 'cozy-client'
import { FluidType } from 'enums'
import { DateTime } from 'luxon'
import { FluidPrice } from 'models'
import mockClient from 'tests/__mocks__/client.mock'
import {
  allLastFluidPrices,
  fluidPrices,
} from 'tests/__mocks__/fluidPrice.mock'
import { getError } from 'tests/__mocks__/testUtils'
import FluidPricesService from './fluidsPrices.service'

describe('FluidPrices service', () => {
  const fluidPricesService = new FluidPricesService(mockClient)

  describe('Fluid Prices - getPrices', () => {
    it('should getPrices for elec', async () => {
      const mockQueryResult: QueryResult<FluidPrice[]> = {
        data: [fluidPrices[0]],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const prices = await fluidPricesService.getPrices(
        FluidType.ELECTRICITY,
        DateTime.fromISO('2021-01-01T00:00:00.000Z', {
          zone: 'utc',
        })
      )
      expect(prices).toBe(fluidPrices[0])
      expect(mockClient.query).toHaveBeenCalled()
    })
    it('should getPrices for gas', async () => {
      const mockQueryResult: QueryResult<FluidPrice[]> = {
        data: [fluidPrices[3]],
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const prices = await fluidPricesService.getPrices(
        FluidType.GAS,
        DateTime.fromISO('2021-10-11T00:00:00.000Z', {
          zone: 'utc',
        })
      )
      expect(prices).toBe(fluidPrices[3])
      expect(mockClient.query).toHaveBeenCalled()
    })
  })

  describe('Fluid Prices - getAllLastPrices', () => {
    it('should getAllLastPrice', async () => {
      const mockQueryResult: QueryResult<FluidPrice[]> = {
        data: fluidPrices,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const prices = await fluidPricesService.getAllLastPrices()
      expect(prices).toStrictEqual(allLastFluidPrices)
      expect(mockClient.query).toHaveBeenCalled()
    })
  })

  it('should checkIfPriceExists and return it', async () => {
    const mockQueryResult: QueryResult<FluidPrice[]> = {
      data: [fluidPrices[0]],
      bookmark: '',
      next: false,
      skip: 0,
    }
    mockClient.query.mockResolvedValueOnce(mockQueryResult)
    const price = await fluidPricesService.checkIfPriceExists(fluidPrices[0])
    expect(price).toStrictEqual(fluidPrices[0])
    expect(mockClient.query).toHaveBeenCalled()
  })
  it('should create a price and return it', async () => {
    const mockQueryResult: QueryResult<FluidPrice> = {
      data: fluidPrices[0],
      bookmark: '',
      next: false,
      skip: 0,
    }
    mockClient.create.mockResolvedValueOnce(mockQueryResult)
    const result = await fluidPricesService.createPrice(fluidPrices[0])
    expect(result).toEqual(fluidPrices[0])
  })
  it('should fail to create a price', async () => {
    mockClient.create.mockRejectedValue(new Error())
    const error = await getError(
      async () => await fluidPricesService.createPrice(fluidPrices[0])
    )
    expect(error).toEqual(new Error())
  })
  it('should update a price', async () => {
    const updatedPrice = { ...fluidPrices[0], price: 0.1 }
    const mockQueryResult: QueryResult<FluidPrice | null> = {
      data: updatedPrice,
      bookmark: '',
      next: false,
      skip: 0,
    }
    mockClient.save.mockResolvedValue(mockQueryResult)
    const price = await fluidPricesService.updatePrice(fluidPrices[0], {
      price: 0.1,
    })
    expect(price).toStrictEqual(updatedPrice)
    expect(mockClient.query).toHaveBeenCalled()
  })
  it('should fail to update a price', async () => {
    mockClient.save.mockRejectedValue(new Error())
    const error = await getError(async () =>
      fluidPricesService.updatePrice(fluidPrices[0], {
        price: 0.1,
      })
    )
    expect(error).toEqual(new Error())
  })
})
